# -*- coding: utf-8 -*-

from copy import copy
from tools import trace, timer

class Parameter(object):
    def __init__(self, status = None):
        self._status = status

        self._name = ""
        self._value = ""

    @property
    def name(self):
        return self._name

    @property
    def value(self):
        return self._value

    def __getitem__(self, key):
        if key == "name":
            return self._name
        elif key == "value":
            return self._value

        return None

    def __setitem__(self, key, value):
        if key == "name":
            self._name = str(value)
        elif key == "value":
            self._value = str(value)

        self._status.modified()

    @classmethod
    def from_tuple(cls, data, status):
        new = cls(status = status)
        new["name"] = data[0]
        new["value"] = data[1]

        return new

class SolverParametersList(object):
    def __init__(self, solver_type = None, status = None):
        super(SolverParametersList, self).__init__()

        self._status = status
        self._parameters = list(
            map(
                lambda t: Parameter.from_tuple(t, self._status),
                solver_type.default_parameters()
            )
        )

    def __len__(self):
        return len(self._parameters)

    @property
    def parameters(self):
        return self._parameters.copy()

    def get(self, index):
        return self._parameters[index]

    def set(self, index, new):
        self._parameters[index] = new
        self._status.modified()

    def new(self, index):
        n = Parameter(status = self._status)
        self._parameters.insert(index, n)
        self._status.modified()
        return n

    def insert(self, index, new):
        self._parameters.insert(index, new)
        self._status.modified()

    def delete(self, parameters):
        for parameter in parameters:
            self._parameters.remove(parameter)
        self._status.modified()

    def delete_i(self, indexes):
        parameters = list(
            map(
                lambda x: x[1],
                filter(
                    lambda x: x[0] in indexes,
                    enumerate(self._parameters)
                )
            )
        )
        self.delete(parameters)

    def sort(self, reverse=False, key=None):
        self._parameters.sort(reverse=reverse, key=key)
        self._status.modified()

    def move_up(self, index):
        if index < len(self._parameters):
            next = index - 1

            l = self._parameters
            l[index], l[next] = l[next], l[index]
            self._status.modified()

    def move_down(self, index):
        if index >= 0:
            prev = index + 1

            l = self._parameters
            l[index], l[prev] = l[prev], l[index]
            self._status.modified()
