# -*- coding: utf-8 -*-

from tools import trace, timer

from View.ASubWindow import ASubWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
)

_translate = QCoreApplication.translate

class SelectSolverWindow(ASubWindow, ListedSubWindow):
    def __init__(self, title="Select solver",
                 study=None, config=None,
                 parent=None):
        self._title = title

        self._study = study
        self._config = config
        self._solver = None

        super(SelectSolverWindow, self).__init__(
            name=self._title, ui="SelectSolver", parent=parent
        )
        self.ui.setWindowTitle(self._title)

        self.setup_combobox()
        self.setup_connections()

    def setup_combobox(self):
        solvers = self._config.solvers
        solvers_name = list(map(lambda s: s.name, solvers))

        self.combobox_add_items("comboBox", solvers_name)

    def setup_connections(self):
        self.find(QPushButton, "pushButton_run").clicked.connect(self.accept)
        self.find(QPushButton, "pushButton_cancel").clicked.connect(self.reject)

    @property
    def solver(self):
        return self._solver

    def accept(self):
        solver_name = self.get_combobox_text("comboBox")
        self._solver = next(
            filter(
                lambda s: s.name == solver_name,
                self._config.solvers
            )
        )

        super(SelectSolverWindow, self).accept()
